'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var solidJs = require('solid-js');
var view = require('@codemirror/view');
var state = require('@codemirror/state');

/**
 * Creates a CodeMirror editor view instance (the editor's user interface).
 * @param props See {@link CodeMirrorProps} for details.
 * @param ref the element to attach the editor to on creation.
 */
function createCodeMirror(props, ref) {
  let view$1;
  solidJs.onMount(() => {
    const state$1 = state.EditorState.create({
      doc: props.value
    }); // Construct a new EditorView instance

    view$1 = new view.EditorView({
      state: state$1,
      parent: ref(),
      dispatch: tr => {
        if (!view$1) return;
        view$1.update([tr]);

        if (tr.docChanged) {
          const newCode = tr.newDoc.sliceString(0, tr.newDoc.length);
          props.onValueChange?.(newCode);
        }
      }
    });
    props.onEditorMount?.(view$1);
    solidJs.onCleanup(() => {
      if (!view$1) return;
      view$1.destroy();
    });
  });
  solidJs.createEffect(solidJs.on(() => props.value, value => {
    if (!view$1 || value === view$1.state.doc.toString()) {
      return;
    }

    view$1.dispatch({
      changes: {
        from: 0,
        to: view$1.state.doc.length,
        insert: value
      }
    });
  }, {
    defer: true
  }));
  /**
   * Creates a compartment instance with the given extension and appends it to the top-level configuration of the editor.
   * See {@link https://codemirror.net/examples/config/| CodeMirror Configuration} and {@link https://codemirror.net/docs/ref/#state.Compartment| Compartment} for details on editor configuration.
   * @param extension the extension to append
   */

  function createExtension(extension) {
    const compartment = new state.Compartment();
    solidJs.onMount(() => {
      if (!view$1) return;
      view$1.dispatch({
        effects: state.StateEffect.appendConfig.of(compartment.of(extension))
      });
    });
    /**
     * Reconfigures the extension compartment with the given extension.
     * @param extension the extension to reconfigure the extension compartment with.
     */

    function reconfigure(extension) {
      if (!view$1) return;
      view$1.dispatch({
        effects: compartment.reconfigure(extension)
      });
    }

    return reconfigure;
  }

  return {
    createExtension
  };
}

exports.createCodeMirror = createCodeMirror;
//# sourceMappingURL=index.common.js.map
